import axios from "axios"
import { createCanvas, loadImage } from "canvas"
import GaussianImgUtil from "../../../utils/gaussianImgUtil"
import localLog from "../../../utils/logUtil"

const path = require("path")
// const fs = require("fs")

const cfgFileName = process.env.NODE_ENV_CFG || "development"

// eslint-disable-next-line import/no-dynamic-require
const dataJSON = require(`../../../config/${cfgFileName}.json`)
localLog(
    "[server]-[config]-[db]-p2 env ",
    String(process.env.NODE_ENV),
    ", cfgFileName ",
    cfgFileName
)
export type MovieImageInfo = {
    imgBytes?: Buffer
    actor: string
    country: string
    description: string
    director: string
    downloadUrl: string
    duration: string
    genre: string
    iconUrl: string
    language: string
    movieUrl: string
    name: string
    score: string
}
/**
 * 电影图片合成
 * @param { 电影Id } mvId
 * @param { 语言? } lang
 * @returns 合成图片的buffer: imgBytes; 电影信息的各个解构字段: ...mvInfo;
 */
export async function movieImageComposition(
    mvId: string,
    lang: string
): Promise<MovieImageInfo | undefined> {
    console.log("movieImageComposition start:")

    // 1. 获取源图片
    let srcImgUrl = ""
    const res = await axios.get(
        `${dataJSON.metaImgActionUrl}/feed/movie/detail?movieId=${mvId}&lang=id&countryCode=ID&region=ID`
    )
    const rlt = res.data
    const mvInfo: MovieImageInfo = {
        actor: "",
        country: "",
        description: "",
        director: "",
        downloadUrl: "",
        duration: "",
        genre: "",
        iconUrl: "",
        language: "",
        movieUrl: "",
        name: "",
        score: "",
    }
    if (rlt.Result === 1 && rlt.data && rlt.data.iconUrl) {
        localLog("correct mv info")
        srcImgUrl = rlt.data.iconUrl
        Object.assign(mvInfo, rlt.data)
        localLog(`getFeedMovieDetail str.length =>${JSON.stringify(rlt.data).length}`)
    } else {
        localLog("wrong mv info")
        return Promise.resolve(undefined)
    }

    // 2. 初始化合成器
    const tarWidth = 806
    const tarHeight = 390

    const image = await loadImage(srcImgUrl)
    const { width, height } = image // 获取图片宽高
    const t_canvas = createCanvas(tarWidth, tarHeight) // 按照微信官方要求，长宽比5:4
    const context = t_canvas.getContext("2d")

    // 3. 使用源图片合成合成新图片
    // 3.1 绘制模糊图
    await GaussianImgUtil.setGaussianBlurCanvas3({
        cvsContext: context,
        tarWidth,
        tarHeight,
        image,
    })
    context.fillStyle = "rgba(0,0,0,.3)"
    context.fillRect(0, 0, tarWidth, tarHeight)

    // 3.2 绘制清晰图
    const dw_img2 = Math.floor((width * 333) / height)
    const dh_img2 = 333
    const dx_img2 = 50
    const dy_img2 = 31
    context.globalAlpha = 1 // 调节透明度
    context.drawImage(image, dx_img2, dy_img2, dw_img2, dh_img2)
    // 黑色透明图层

    // 3.3 打印评分
    const scorebgImgPath = path.resolve(__dirname, "../../../../public/img/share/score_bg.png")
    const scorebgImg = await loadImage(scorebgImgPath)
    context.drawImage(scorebgImg, 206, 330, scorebgImg.width * 2, scorebgImg.height * 2)

    context.font = "22px bold" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "#FFC63D"
    context.fillText(Number(mvInfo.score).toFixed(1), 215, 351)

    // 3.4 绘制logo
    const logoImgPath = path.resolve(__dirname, "../../../../public/img/share/logo.png")
    const logoImg = await loadImage(logoImgPath)
    context.drawImage(logoImg, 309, 46, logoImg.width, logoImg.height)

    // 3.5 打印title
    context.font = "30px bold Arial" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "white"
    const nameTxtOffX = 309
    const nameTxtWidth = context.measureText(mvInfo.name).width
    const nameTxtSubLen = Math.ceil(
        ((tarWidth - nameTxtOffX - 48) / nameTxtWidth) * mvInfo.name.length
    )
    if (nameTxtSubLen < mvInfo.name.length) {
        context.fillText(`${mvInfo.name.substr(0, nameTxtSubLen)}...`, nameTxtOffX, 140)
    } else {
        context.fillText(mvInfo.name, nameTxtOffX, 140)
    }

    // 3.6 打印国家/语言
    context.font = "30px 'Microsoft YaHei'" // 统一使用微软雅黑字体  "Microsoft YaHei"
    context.fillStyle = "#ffffffb2"
    const countryLang =
        mvInfo.country.indexOf("/") > -1
            ? mvInfo.country
            : mvInfo.country + (mvInfo.language ? ` / ${mvInfo.language}` : "")
    context.fillText(countryLang, 309, 195)

    // 3.7 打印类型
    // context.font = '24px "Microsoft YaHei"' // 统一使用微软雅黑字体
    const genreTxtOffX = 309
    const genreTxtWidth = context.measureText(mvInfo.genre).width
    const genreTxtSubLen = Math.ceil(
        ((tarWidth - genreTxtOffX - 48) / genreTxtWidth) * mvInfo.genre.length
    )
    if (genreTxtSubLen < mvInfo.genre.length) {
        context.fillText(`${mvInfo.genre.substr(0, genreTxtSubLen)}...`, genreTxtOffX, 235)
    } else {
        context.fillText(mvInfo.genre, genreTxtOffX, 235)
    }

    // 3.8 打印演员: 42 chars limit, 两行，第二行省略号
    // context.font = '24px "Microsoft YaHei"' // 统一使用微软雅黑字体
    const actor = (lang === "id" ? "Aktor: " : "Actor: ") + mvInfo.actor
    let actorPcs = actor.split(" ")
    for (let i = 0; i < actorPcs.length; i += 1) {
        const pcs2 = actorPcs[i].split(",")
        actorPcs[i] = pcs2.join(", ")
    }
    actorPcs = actorPcs.join(" ").split(" ") // 分词换行处理
    // 第一行
    let actorTxtOffXP1 = 309
    let idx = 0 // 最多1000个词
    while (actorPcs.length > 0 && idx < 1000) {
        const word = `${actorPcs.shift()} `
        const wordW = context.measureText(word).width
        if (actorTxtOffXP1 + wordW > tarWidth - 48) {
            break
        } else {
            context.fillText(word, actorTxtOffXP1, 285)
        }

        actorTxtOffXP1 += wordW
        idx += 1
    }
    // 第二行
    let actorTxtOffXP2 = 309
    idx = 0 // 最多1000个词
    while (actorPcs.length > 0 && idx < 1000) {
        const word = `${actorPcs.shift()} `
        const wordW = context.measureText(word).width
        if (actorTxtOffXP2 + wordW > tarWidth - 48) {
            context.fillText("...", actorTxtOffXP2, 325)
            break
        } else {
            context.fillText(word, actorTxtOffXP2, 325)
        }

        actorTxtOffXP2 += wordW
        idx += 1
    }

    // 4. 生成文件字节流
    const file = t_canvas.toDataURL("image/png")
    const data2 = file.replace(/^data:image\/\w+;base64,/, "")
    const imgBytes = Buffer.from(data2, "base64")

    // //本地保存，测试代码
    // const dataBuffer = new Buffer(data2, 'base64');
    // fs.writeFile(path.resolve(__dirname, '../../../../public/img/share/out.png'), dataBuffer, function(err){
    //   if (err) {
    //     console.log('保存失败！', err)
    //   } else {
    //     console.log("保存成功！");
    //   }
    // });

    localLog("movieImageComposition end.", String(imgBytes.length))
    return Promise.resolve({
        imgBytes,
        ...mvInfo,
    })
}
